--mod to provide additional challenge for ship landing part of the game
require "defines"
require "util"
--variables
--[[
glob.attackdata=
	{
		cap - max number of biters sent
		timer - time in ticks until next attack
		interval - interval between attacks
	}
glob.target  - entity which attracts biters, usually it'll be the rocket defense
glob.bases=
	{--a table of biter bases from which units are going to be sent
		{x,y} - approximate position of biter base
	}
]]
------------------------------------------------------------------------
--few commands which might be used in analogy with "freeplay" ones
remote.addinterface("doomsday",
{
  setattackdata = function(data)--function to modify attackdata table
--allows incomplete definition of fields. Undefined ones will keep their previous
--values.    
	for index in pairs(glob.attackdata) do 
		if data[index]~=nil  then
			glob.attackdata[index]=data[index]
		end
	end
  end,

  getattackdata = function()
    return glob.attackdata
  end,
})
------------------------------------------------------------------------
game.oninit(function()
	initattackdata()
end)

game.onload(function()
	if glob.attackdata==nil then initattackdata() end
end)

--this gives biters the target
game.onevent({defines.events.onbuiltentity,onrobotbuiltentity},
function(event)
	if event.createdentity.name=="rocket-defense" then
		glob.target=event.createdentity
	end
	--game.player.print(serpent.block(glob.target.position))
end)

--this makes you lose if rocket defense ceases being after the landing has begun
game.onevent({defines.events.onrobotpremined, defines.events.onpreplayermineditem,defines.events.onentitydied},
function(event)
		if event.entity.name=="rocket-defense" and game.player.force.getshiplandinginprogress() then
			game.setgamestate{gamefinished=true, playerwon=false}
		end
end)

--this launches the main loop
game.onevent(defines.events.onshiplandingstart, 
function(event)
--prepare the biter plan for the battle
	initbases()
	game.evolutionfactor=1
--subscribe the attack function
	game.onevent(defines.events.ontick,function(event)
		glob.attackdata.timer=glob.attackdata.timer-1
		if glob.attackdata.timer<=0 then
			if game.ispathfinderbusy() then 
				glob.attackdata.timer=60
				--game.player.print("another time")
			else 
				send_biters() 
				glob.attackdata.timer=glob.attackdata.interval
			end
		end
	end)
end)


------------------------------------------------------------------------
function initattackdata()
	glob.attackdata=
	{cap=100, timer=1, interval=30*30}
	glob.bases={}
end

function chunk_to_tiles(chunk) --transform chunk coords to coords of it's middle tile
	local tile={}
	tile.x=math.floor((chunk.x+0.5)*32)
	tile.y=math.floor((chunk.y+0.5)*32)
	return tile
end

function initbases()
	--check all the generated chunks
	local amount=0
	for coord in game.getchunks() do
		amount=amount+1
		if game.ischunkgenerated(coord) then 
			coord=chunk_to_tiles(coord)
			--game.player.force.chart{{coord.x-16, coord.y-16},{coord.x+16, coord.y+16}}
			--for the given chunk check if there are biters 
			local biters=game.findenemyunits(coord, 16)
			if #biters>0 then
				--see if they're there because there's their base
				if #(game.findentitiesfiltered{area={{coord.x-16, coord.y-16},{coord.x+16, coord.y+16}},
				type = "unit-spawner"})>0 then--this assumes that only enemy spawners can exist there
				--if so, mark it's location
					table.insert(glob.bases,coord)
				end
			end
 		end
 		--if amount>100 then break end
	end
	--sort the array, so biters are sent from the closest bases
	table.sort(glob.bases,function(a,b)	return closest(a,b,glob.target.position) end)
	table.sort(glob.bases,function(a,b) return closest(a,b,glob.bases[1]) end)
	
end

function send_biters()
	--we just go around their bases
	local amount_bit=0
	for _,base in ipairs(glob.bases) do
	--and put them in squads
		local group=game.createunitgroup{position=base}
		for _, biter in ipairs(game.findenemyunits(base, 16)) do
			if not biter.hascommand() then--we also see if they aren't in other squad already
				group.addmember(biter)
				amount_bit=amount_bit+1
				--game.player.print("found someone")
			end
		end
		--then we see if there were any
		if #group.members==0 then
			group.destroy()
			--game.player.print("broke group")
		else--if there were, send them to war
			group.setcommand{type=defines.command.attack,
					 target=glob.target,
					 distraction=defines.distraction.byanything}
			--game.player.force.chart{{base.x-16, base.y-16},{base.x+16, base.y+16}}
		end
		--see if we have sent to die enough
		if amount_bit>glob.attackdata.cap then
			break
		end
	end
	--game.player.print("sent "..amount_bit)
end

function closest(a,b,c)--returns true if position "a" is closer to "c" than the "b"
	local dist_a=(a.x-c.x)^2+(a.y-c.y)^2
	local dist_b=(b.x-c.x)^2+(b.y-c.y)^2
	return dist_a<dist_b
end
